@@ -7,9 +7,12 @@ __version__ = get_version(__file__)
 del get_version
 
 from . import pp
+from . import est
 from . import tl
+from . import vf
 from . import pd
 from . import pl
+from . import mv
 from . import sim
 from .data_io import *
 from . import sample_data
