@@ -13,9 +13,9 @@ from .utils import (
 from ..tools.dimension_reduction import reduceDimension
 from ..tools.cell_vectors import cell_velocities
 from ..tools.Markov import prepare_velocity_grid_data, velocity_on_grid, grid_velocity_filter
-from ..tools.topography import VectorField
+from ..vectorfield.topography import VectorField
 from ..tools.utils import update_dict
-from ..tools.utils_vecCalc import vecfld_from_adata
+from ..vectorfield.utils_vecCalc import vecfld_from_adata
 
 from .scatters import docstrings
 
@@ -979,7 +979,7 @@ def streamline_plot(
     }
     mass = np.sqrt((V_grid ** 2).sum(0))
     linewidth *= 2 * mass / mass[~np.isnan(mass)].max()
-    streamplot_kwargs.update({"linewidth": linewidth})
+    streamplot_kwargs.update({"linewidth": linewidth * streamline_kwargs.pop('linewidth', 1)})
 
     streamplot_kwargs = update_dict(streamplot_kwargs, streamline_kwargs)
 
