@@ -1,4 +1,3 @@
-from tqdm import tqdm
 import warnings
 import scipy as scp
 from scipy.sparse import csr_matrix, issparse
@@ -6,8 +5,8 @@ from sklearn.decomposition import PCA
 from sklearn.utils import sparsefuncs
 from .Markov import *
 from .connectivity import extract_indices_dist_from_graph
-from .topography import VectorField
-from .vector_calculus import acceleration
+
+from .metric_velocity import gene_wise_confidence
 from .utils import (
     set_velocity_genes,
     get_finite_inds,
@@ -16,7 +15,7 @@ from .utils import (
     update_dict,
     get_iterative_indices,
     split_velocity_graph,
-    norm_row,
+    norm,
     einsum_correlation,
     build_distance_graph,
     log1p_,
@@ -185,21 +184,31 @@ def cell_velocities(
                 adata.uns["neighbors"]["indices"],
             )
             indices, dist = indices[:, 1:], dist[:, 1:]
+    else:
+        raise Exception(f"Seems like your adata object doesn't have neighbor information. "
+                        "Try running `dyn.tl.reduceDimension` or `dyn.tl.neighbors` first.")
 
+    if 'confident_gene' in adata.var.keys() and not enforce:
+        X = adata[:, adata.var.confident_gene.values].layers[ekey] if X is None else X
+        V_mat = (
+            adata[:, adata.var.confident_gene.values].layers[vkey]
+            if vkey in adata.layers.keys()
+            else None
+        ) if V_mat is None else V_mat
+    else:
+        if 'use_for_velocity' not in adata.var.keys() or enforce:
+            use_for_dynamics = True if "use_for_dynamics" in adata.var.keys() else False
+            adata = set_velocity_genes(
+                adata, vkey="velocity_S", min_r2=min_r2, use_for_dynamics=use_for_dynamics,
+                min_alpha=min_alpha, min_gamma=min_gamma, min_delta=min_delta,
+            )
 
-    if 'use_for_velocity' not in adata.var.keys() or enforce:
-        use_for_dynamics = True if "use_for_dynamics" in adata.var.keys() else False
-        adata = set_velocity_genes(
-            adata, vkey="velocity_S", min_r2=min_r2, use_for_dynamics=use_for_dynamics,
-            min_alpha=min_alpha, min_gamma=min_gamma, min_delta=min_delta,
-        )
-
-    X = adata[:, adata.var.use_for_velocity.values].layers[ekey] if X is None else X
-    V_mat = (
-        adata[:, adata.var.use_for_velocity.values].layers[vkey]
-        if vkey in adata.layers.keys()
-        else None
-    ) if V_mat is None else V_mat
+        X = adata[:, adata.var.use_for_velocity.values].layers[ekey] if X is None else X
+        V_mat = (
+            adata[:, adata.var.use_for_velocity.values].layers[vkey]
+            if vkey in adata.layers.keys()
+            else None
+        ) if V_mat is None else V_mat
 
     if X.shape != V_mat.shape and X.shape[0] != adata.n_obs:
         raise Exception(f"X and V_mat doesn't have the same dimensionalities or X/V_mat doesn't {adata.n_obs} rows!")
@@ -258,7 +267,7 @@ def cell_velocities(
             from pynndescent import NNDescent
 
             nbrs = NNDescent(X, metric='eulcidean', n_neighbors=30, n_jobs=-1,
-                              random_state=19490110, **kwargs)
+                              random_state=19490110)
             indices, _ = nbrs.query(X, k=30)
         else:
             alg = "ball_tree" if X.shape[1] > 10 else 'kd_tree'
@@ -422,98 +431,85 @@ def cell_velocities(
     return adata
 
 
-def cell_accelerations(adata,
-                       vf_basis='pca',
-                       basis='umap',
-                       enforce=True,
-                       preserve_len=True,
-                       other_kernels_dict={},
-                       **kwargs):
-    """Compute transition probability and project high dimension acceleration vector to existing low dimension embedding.
+def confident_cell_velocities(adata,
+                            group,
+                            progenitors_groups,
+                            mature_cells_groups,
+                            ekey='M_s',
+                            vkey='velocity_S',
+                            basis='umap',
+                            confidence_threshold=0.85,
+                            only_velocity_genes=False,
+                            ):
+    """Confidently compute transition probability and project high dimension velocity vector to existing low dimension
+    embeddings using progeintors and mature cell groups priors.
 
-    In classical physics, including fluidics and aerodynamics, velocity and acceleration vector fields are used as
-    fundamental tools to describe motion or external force of objects, respectively. In analogy, RNA velocity or
-    accelerations estimated from single cells can be regarded as samples in the velocity (La Manno et al. 2018) or
-    acceleration vector field (Gorin, Svensson, and Pachter 2019). In general, a vector field can be defined as a
-    vector-valued function f that maps any points (or cells’ expression state) x in a domain Ω with D dimension (or the
-    gene expression system with D transcripts / proteins) to a vector y (for example, the velocity or acceleration for
-    different genes or proteins), that is f(x) = y.
-
-    In two or three dimensions, a velocity vector field is often visualised as a quiver plot where a collection of arrows
-    with a given magnitude and direction is drawn. For example, the velocity estimates of unspliced transcriptome of
-    sampled cells projected into two dimensions is drawn to show the prediction of the future cell states in RNA velocity
-    (La Manno et al. 2018). During the differentiation process, external signal factors perturb cells and thus change
-    the vector field. Since we perform genome-wide profiling of cell states and the experiments performed are often done
-    in a short time scale, we assume a constant vector field without loss of generality (See also Discussion). Assuming
-    an asymptotic deterministic system, the trajectory of the cells travelling in the gene expression space follows the
-    vector field and can be calculated using numerical integration methods, for example Runge-Kutta algorithm. In two or
-    three dimensions, a streamline plot can be used to visualize the paths of cells will follow if released in different
-    regions of the gene expression state space under a steady flow field. Another more intuitive way to visualize the
-    structure of vector field is the so called line integral convolution method or LIC (Cabral and Leedom 1993), which
-    works by adding random black-and-white paint sources on the vector field and letting the flowing particle on the
-    vector field picking up some texture to ensure the same streamline having similar intensity. Although we have not
-    provides such functionalities in dynamo, with vector field that changes over time, similar methods, for example,
-    streakline, pathline, timeline, etc. can be used to visualize the evolution of single cell or cell populations.
-
-    Arguments
-    ---------
+    Parameters
+    ----------
         adata: :class:`~anndata.AnnData`
-            an Annodata object.
-        vf_basis: 'int' (optional, default `pca`)
-            The dictionary key that corresponds to the low dimensional embedding where the vector field function
-            reconstructed.
+            an Annodata object
+        group: `str`
+            The column key/name that identifies the cell state grouping information of cells. This will be used for
+            calculating gene-wise confidence score in each cell state.
+        progenitors_groups: `list`
+            The group names from `group` that corresponding to the states of progenitors.
+        mature_cells_groups: `list`
+            The group names from `group` that corresponding to the states of terminal cell states. The best practice for
+            determining terminal cell states are those fully functional cells instead of intermediate cell states.
+        ekey: `str` or None (default: `M_s`)
+            The layer that will be used to retrieve data for identifying the gene is in induction or repression phase at
+            each cell state. If `None`, .X is used.
+        vkey: `str` or None (default: `velocity_S`)
+            The layer that will be used to retrieve velocity data for calculating gene-wise confidence. If `None`,
+            `velocity_S` is used.
         basis: 'int' (optional, default `umap`)
             The dictionary key that corresponds to the reduced dimension in `.obsm` attribute.
-        enforce: `bool` (default: `False`)
-            Whether to enforce 1) redefining use_for_velocity column in obs attribute;
-                               2) recalculation of transition matrix.
-        preserve_len: `bool` (default: `True`)
-            Whether to preserve the length of high dimension vector length. When set to be True, the length  of low
-            dimension projected vector will be proportionally scaled to that of the high dimensional vector. Note that
-            when preserve_len is set to be True, the acceleration field may seem to be messy (although the magnitude will
-            be reflected) while the trend of acceleration when `preserve_len` is `True` is more clearer but will lose
-            information of acceleration magnitude. This is because the acceleration is not directly related to the
-            distance of cells in the low embedding space; thus the acceleration direction can be better preserved than
-            the magnitude. On the other hand, velocity is more relevant to the distance in low embedding space, so
-            preserving magnitude and direction of velocity vector in low dimension can be more easily achieved.
-        other_kernels_dict: `dict` (default: `{}`)
-            A dictionary of paramters that will be passed to the cosine/correlation kernel.
+        confidence_threshold: 'float' (optional, default `0.85`)
+            The minimal threshold of the mean of the average progenitors and the average mature cells prior based
+            gene-wise score. Only genes with score larger than this will be considered as confident velocity genes for
+            velocity projection.
+        only_velocity_genes: 'bool' (optional, default `False`)
+            Whether only use previous identified velocity genes for confident gene selection, followed by velocity
+            projection.
 
     Returns
     -------
         Adata: :class:`~anndata.AnnData`
-            Returns an updated `~anndata.AnnData` with transition_matrix and projected embedding of high dimension velocity
-            vectors in the existing embeddings of current cell state, calculated using either the Itô kernel method
-            (default) or the diffusion approximation or the method from (La Manno et al. 2018).
+            Returns an updated `~anndata.AnnData` with only confident genes based transition_matrix and projected
+            embedding of high dimension velocity vectors in the existing embeddings of current cell state, calculated
+            using either the Itô kernel method (default) or the diffusion approximation or the method from
+            (La Manno et al. 2018).
     """
 
-    if 'velocity_' + vf_basis not in adata.obsm.keys():
-        cell_velocities(adata, basis=vf_basis)
+    if not any([i.startswith('velocity') for i in adata.layers.keys()]):
+        raise Exception(f"You need to first run `dyn.tl.dynamics(adata)` to estimate kinetic parameters and obtain "
+                        f"raw RNA velocity before running this function.")
 
-    if 'VecFld_' + vf_basis not in adata.uns_keys():
-        VectorField(adata, basis=vf_basis)
+    if only_velocity_genes:
+        if 'use_for_velocity' not in adata.var.keys():
+            warnings.warn('`dyn.tl.cell_velocities(adata)` is not performed yet. Rolling back to use all feature genes '
+                          'as input for supervised RNA velocity analysis.')
+            genes = adata.var_names[adata.var.use_for_dynamics]
+        else:
+            genes = adata.var_names[adata.var.use_for_velocity]
+    else:
+        genes = adata.var_names[adata.var.use_for_dynamics]
 
-    if 'acceleration_' + vf_basis not in adata.obsm.keys():
-        acceleration(adata, basis=vf_basis)
+    gene_wise_confidence(adata, group, progenitors_groups, mature_cells_groups, genes=genes, layer=ekey, vlayer=vkey,)
 
-    X = adata.obsm['X_' + vf_basis]
-    V_mat = adata.obsm['acceleration_' + vf_basis]
+    adata.var.loc[:, 'avg_confidence'] = (adata.var.loc[:, 'avg_prog_confidence'] +
+                                          adata.var.loc[:, 'avg_mature_confidence']) / 2
+    confident_genes = genes[adata[:, genes].var['avg_confidence'] > confidence_threshold]
+    adata.var['confident_genes'] = False
+    adata.var.loc[confident_genes, 'confident_genes'] = True
+
+    X = adata[:, confident_genes].layers[ekey]
+    V_mat = adata[:, confident_genes].layers[vkey]
     X_embedding = adata.obsm['X_' + basis]
 
-    if basis != vf_basis and vf_basis.lower() not in ['umap', 'tsne', 'trimap', 'ddtree', 'diffusion_map']:
-        cell_velocities(
-            adata,
-            X=X,
-            V_mat=V_mat,
-            X_embedding=X_embedding,
-            basis=basis,
-            enforce=enforce,
-            key='acceleration',
-            preserve_len=preserve_len,
-            other_kernels_dict=other_kernels_dict,
-            **kwargs
-        )
+    cell_velocities(adata, enforce=True, X=X, V_mat=V_mat, X_embedding=X_embedding, basis=basis)
 
+    return adata
 
 def stationary_distribution(adata, method="kmc", direction="both", calc_rnd=True):
     """Compute stationary distribution of cells using the transition matrix.
@@ -802,7 +798,7 @@ def projection_with_transition_matrix(n, T, X_embedding):
         for i in tqdm(range(n), desc=f"projecting velocity vector to low dimensional embedding..."):
             idx = T[i].indices
             diff_emb = X_embedding[idx] - X_embedding[i, None]
-            diff_emb /= norm_row(diff_emb)[:, None]
+            diff_emb /= norm(diff_emb, axis=1)[:, None]
             diff_emb[np.isnan(diff_emb)] = 0
             T_i = T[i].data
             delta_X[i] = T_i.dot(diff_emb) - T_i.mean() * diff_emb.sum(0)
