@@ -3,6 +3,7 @@ import numpy as np
 import pandas as pd
 from scipy import interpolate, stats
 from scipy.sparse import issparse, csr_matrix
+import scipy.sparse.linalg as splinalg
 from scipy.integrate import odeint
 from scipy.linalg.blas import dgemm
 from sklearn.neighbors import NearestNeighbors
@@ -91,6 +92,10 @@ def flatten(arr):
 
 
 def isarray(arr):
+    """
+        Check if a variable is an array. Essentially the variable has the attribute 'len'
+        and it is not a string.
+    """
     return hasattr(arr, '__len__') and (not isinstance(arr, str))
 
 
@@ -110,16 +115,12 @@ def elem_prod(X, Y):
         return np.multiply(X, Y)
 
 
-def norm_vector(x):
-    """calculate euclidean norm for a row vector"""
-
-    return np.sqrt(np.einsum('i, i -> ', x, x))
-
-
-def norm_row(X):
-    """calculate euclidean norm for each row of a matrix"""
-
-    return np.sqrt(X.multiply(X).sum(1).A1 if issparse(X) else np.einsum('ij, ij -> i', X, X) if X.ndim > 1 else np.einsum('i, i -> ', X, X))
+def norm(x, **kwargs):
+    """calculate the norm of an array or matrix"""
+    if issparse(x):
+        return splinalg.norm(x, **kwargs)
+    else:
+        return np.linalg.norm(x, **kwargs)
 
 
 def einsum_correlation(X, Y_i, type="pearson"):
@@ -137,7 +138,7 @@ def einsum_correlation(X, Y_i, type="pearson"):
         corr = np.array([stats.kendalltau(x, Y_i)[0] for x in X])
         return corr[None, :]
 
-    X_norm, Y_norm = norm_row(X),  norm_vector(Y_i)
+    X_norm, Y_norm = norm(X, axis=1), norm(Y_i)
 
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
@@ -166,6 +167,7 @@ def form_triu_matrix(arr):
                     break
     return M
 
+
 def index_condensed_matrix(n, i, j):
     """
     Return the index of an element in a condensed n-by-n square matrix 
@@ -193,11 +195,6 @@ def index_condensed_matrix(n, i, j):
     return int(i * (n - (i + 3) * 0.5) + j - 1)
 
 
-def is_outside_domain(x, domain):
-    x = x[None, :] if x.ndim == 1 else x
-    return np.any(np.logical_or(x < domain[0], x > domain[1]), axis=1)
-
-
 def moms2var(m1, m2):
     var = m2 - elem_prod(m1, m1)
     return var
@@ -248,7 +245,6 @@ def velocity_on_grid(X, V, n_grids, nbrs=None, k=None,
         k = 100 if k is None else k
         if X.shape[0] > 200000 and X.shape[1] > 2: 
             from pynndescent import NNDescent
-
             nbrs = NNDescent(X, metric='euclidean', n_neighbors=k+1, n_jobs=-1, random_state=19491001)
         else:
             alg = 'ball_tree' if X.shape[1] > 10 else 'kd_tree'
@@ -275,6 +271,21 @@ def velocity_on_grid(X, V, n_grids, nbrs=None, k=None,
     return X_grid, V_grid
 
 
+def list_top_genes(arr, gene_names, n_top_genes=30, order=-1):
+    imax = np.argsort(arr)[::order]
+    return gene_names[imax][:n_top_genes]
+
+
+def table_top_genes(arrs, item_names, gene_names, return_df=True, **kwargs):
+    table = {}
+    for i, item in enumerate(item_names):
+        table[item] = list_top_genes(arrs[i], gene_names, **kwargs)
+    if return_df:
+        return pd.DataFrame(data=table)
+    else:
+        return table
+
+
 # ---------------------------------------------------------------------------------------------------
 # data transformation related:
 def log1p_(adata, X_data):
@@ -318,6 +329,21 @@ def inverse_norm(adata, layer_x):
 
     return layer_x
 
+
+# ---------------------------------------------------------------------------------------------------
+# neighbors related:
+def get_conn_dist_graph(knn, distances):
+    n_obs, n_neighbors = knn.shape
+    distances = csr_matrix((distances.flatten(), (np.repeat(np.arange(n_obs), n_neighbors), knn.flatten())),
+                           shape=(n_obs, n_obs))
+    connectivities = distances.copy()
+    connectivities.data[connectivities.data > 0] = 1
+
+    distances.eliminate_zeros()
+    connectivities.eliminate_zeros()
+
+    return distances, connectivities
+
 # ---------------------------------------------------------------------------------------------------
 # dynamics related:
 def one_shot_gamma_alpha(k, t, l):
